<?php

/*
 * Define some basic events for drupal core
 */

/*
 * Implementation of hook_event_info()
 */
function workflow_ng_event_info() {

  $events = array(
    'node_insert' => array(
      '#label' => t('Content has been created'),
      '#module' => t('Node'),
      '#arguments' => workflow_ng_events_node_arguments(t('created content'), t('content\'s author')),
    ),
    'node_update' => array(
      '#label' => t('Content has been updated'),
      '#module' => t('Node'),
      '#arguments' => workflow_ng_events_node_arguments(t('updated content'), t('content author')) + array(
        'node_unchanged' => array('#entity' => 'node', '#label' => t('unchanged content'), '#handler' => 'workflow_ng_events_argument_node_unchanged'),
        'author_unchanged' => array('#entity' => 'user', '#label' => t('unchanged content\'s author'), '#handler' => 'workflow_ng_events_argument_unchanged_node_author'),
      ),
    ),
    'node_submit' => array(
      '#label' => t('Content is going to be saved'),
      '#module' => t('Node'),
      '#arguments' => array(
        'node' => array('#entity' => 'node', '#label' => t('content'), '#saved' => TRUE),
        'author' => array('#entity' => 'user', '#label' => t('content author'), '#handler' => 'workflow_ng_events_argument_node_author'),
        'node_unchanged' => array('#entity' => 'node', '#label' => t('unchanged content'), '#handler' => 'workflow_ng_events_argument_node_unchanged'),
        'author_unchanged' => array('#entity' => 'user', '#label' => t('unchanged content\'s author'), '#handler' => 'workflow_ng_events_argument_unchanged_node_author'),
      ) + workflow_ng_events_global_user_argument(),
    ),
    'node_view' => array(
      '#label' => t('Content is going to be viewed'),
      '#module' => t('Node'),
      '#description' => t('Note that if drupal\'s page cache is enabled, this event won\'t be generated for pages served from cache.'),
      '#arguments' => workflow_ng_events_node_arguments(t('viewed content'), t('content author')),
    ),
    'node_delete' => array(
      '#label' => t('Content has been deleted'),
      '#module' => t('Node'),
      '#arguments' => workflow_ng_events_node_arguments(t('deleted content'), t('content author')),
    ),
    'user_insert' => array(
      '#label' => t('User has registered'),
      '#module' => t('User'),
      '#arguments' => workflow_ng_events_hook_user_arguments(t('registered user')),
    ),
    'user_update' => array(
      '#label' => t('User has updated his account details'),
      '#module' => t('User'),
      '#arguments' => workflow_ng_events_hook_user_arguments(t('updated user')) + array('account_unchanged' => array('#entity' => 'user', '#label' => t('unchanged user'))),
    ),
    'user_view' => array(
      '#label' => t('User page has been viewed'),
      '#module' => t('User'),
      '#description' => t('Note that if drupal\'s page cache is enabled, this event won\'t be generated for pages served from cache.'),
      '#arguments' => workflow_ng_events_hook_user_arguments(t('viewed user')),
    ),
    'user_delete' => array(
      '#label' => t('User has been deleted'),
      '#module' => t('User'),
      '#arguments' => workflow_ng_events_hook_user_arguments(t('deleted user')),
    ),
    'user_login' => array(
      '#label' => t('User has logged in'),
      '#module' => t('User'),
      '#arguments' => array('account' => array('#entity' => 'user', '#label' => t('logged in user'))),
    ),
    'user_logout' => array(
      '#label' => t('User has logged out'),
      '#module' => t('User'),
      '#arguments' => array('account' => array('#entity' => 'user', '#label' => t('logged out user'))),
    ),
    'comment_insert' => array(
      '#label' => t('Comment has been created'),
      '#module' => t('Comment'),
      '#arguments' => workflow_ng_events_hook_comment_arguments('created comment'),
    ),
    'comment_update' => array(
      '#label' => t('Comment has been updated'),
      '#module' => t('Comment'),
      '#arguments' => workflow_ng_events_hook_comment_arguments('updated comment'),
    ),
    'comment_delete' => array(
      '#label' => t('Comment has been deleted'),
      '#module' => t('Comment'),
      '#arguments' => workflow_ng_events_hook_comment_arguments('deleted comment'),
    ),
    'init' => array(
      '#label' => t('User is going to view a page'),
      '#module' => t('System'),
      '#description' => t('Note that if drupal\'s page cache is enabled, this event won\'t be generated for pages served from cache.'),
      '#arguments' => workflow_ng_events_global_user_argument(),
    ),
  );
  return $events;
}

/*
 * Returns the argument definition for the acting user (global $user)
 */
function workflow_ng_events_global_user_argument() {
  return array('user' => array('#entity' => 'user', '#label' => t('acting user'), '#handler' => 'workflow_ng_events_argument_global_user'));
}

/*
 * Returns some arguments suitable for using it with a node
 */
function workflow_ng_events_node_arguments($node_label, $author_label) {
  return array(
    'node' => array('#entity' => 'node', '#label' => $node_label),
    'author' => array('#entity' => 'user', '#label' => $author_label, '#handler' => 'workflow_ng_events_argument_node_author'),
  ) + workflow_ng_events_global_user_argument();
}

/*
 * Returns some arguments suitable for hook_user
 */
function workflow_ng_events_hook_user_arguments($account_label) {
  return array(
    'account' => array('#entity' => 'user', '#label' => $account_label),
  ) + workflow_ng_events_global_user_argument();
}

/*
 * Returns some arguments suitable for hook comment
 */
function workflow_ng_events_hook_comment_arguments($comment_label) {
  return array(
    'comment' => array('#entity' => 'comment', '#label' => t($comment_label)),
    'comment_author' => array('#entity' => 'user', '#label' => t("$comment_label author"), '#handler' => 'workflow_ng_events_argument_comment_author'),
    'node' => array('#entity' => 'node', '#label' => t('commented content'), '#handler' => 'workflow_ng_events_argument_comment_node'),
    'node_author' => array('#entity' => 'user', '#label' => t('commented content author'), '#handler' => 'workflow_ng_events_argument_comment_node_author'),
  ) + workflow_ng_events_global_user_argument();
}

/*
 * Implementation of hook_nodeapi
 */
function workflow_ng_nodeapi(&$node, $op, $teaser = NULL, $page = NULL) {

  if (in_array($op, array('view', 'insert', 'update', 'submit', 'delete'))) {
    workflow_ng_invoke_event('node_'. $op, $node);
  }
}

/*
 * Implementation of hook_user
 */
function workflow_ng_user($op, $edit, $account, $category = NULL) {
  static $account_unchanged;

  //we don't support updates for other categories than 'account'
  if ($op == 'update' && $category != 'account') {
    return;
  }
  else if ($op == 'update') {
    //save the unchanged account for the use with op after_update
    $account_unchanged = $account;
  }
  else if ($op == 'after_update') {
    workflow_ng_invoke_event('user_update', $account, $edit, $account_unchanged);
  }
  else if (in_array($op, array('insert', 'login', 'view', 'delete'))) {
    workflow_ng_invoke_event('user_'. $op, $account, $edit);
  }
}

/*
 * Implementation of hook_comment
 */
function workflow_ng_comment($comment, $op) {
  if (in_array($op, array('insert', 'update', 'delete'))) {
    if (is_array($comment)) {
      $comment = (object)$comment;
    }
    workflow_ng_invoke_event('comment_'. $op, $comment);
  }
}

/*
 * Implementation of hook_menu
 */
function workflow_ng_menu($may_cache) {
  if (!$may_cache) {
    workflow_ng_invoke_event('init');
  }
}

/*
 * Argument handlers
 */

/*
 * Gets the author's account of a node
 */
function workflow_ng_events_argument_node_author($node) {
  return user_load(array('uid' => $node->uid));
}

/*
 * Gets the user account of the "acting" user - which is always the global user
 */
function workflow_ng_events_argument_global_user() {
  global $user;
  return user_load(array('uid' => $user->uid));
}

/*
 * Gets the node object, that doesn't contain the modified changes
 */
function workflow_ng_events_argument_node_unchanged($node) {
  return $node->nid ? node_load($node->nid) : $node;
}

/*
 * Gets the author of the unchanged node object
 */
function workflow_ng_events_argument_unchanged_node_author($node) {
  return workflow_ng_events_argument_node_author(workflow_ng_events_argument_node_unchanged($node));
}

/*
 * Gets the author of the comment
 */
function workflow_ng_events_argument_comment_author($comment) {
  return user_load(array('uid' => $comment->uid));
}

/*
 * Gets the comment's node
 */
function workflow_ng_events_argument_comment_node($comment) {
  return node_load($comment->nid);
}

/*
 * Gets the comment's node's author
 */
function workflow_ng_events_argument_comment_node_author($comment) {
  return workflow_ng_events_argument_node_author(workflow_ng_events_argument_comment_node($comment));
}
